'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.argsToFindOptions = argsToFindOptions;
exports.buildQuery = buildQuery;
exports.buildCount = buildCount;
exports.mapCountToResultSet = mapCountToResultSet;

var _thinkyExportSchema = require('thinky-export-schema');

var _thinkyExportSchema2 = _interopRequireDefault(_thinkyExportSchema);

var _resolver = require('./resolver');

var _lodash = require('lodash');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Args to find options
 *
 * @param args
 * @param model
 * @param opts
 * @returns {{}}
 */
function argsToFindOptions(args, model) {
  var opts = arguments.length <= 2 || arguments[2] === undefined ? { maxLimit: 50 } : arguments[2];

  var result = {
    filter: {},
    limit: undefined,
    skip: undefined,
    order: undefined
  },
      modelSchema = (0, _thinkyExportSchema2.default)(model),
      attributes = Object.keys(modelSchema.fields).concat('id');

  opts.maxLimit = opts.maxLimit === undefined ? false : parseInt(opts.maxLimit, 10);

  if (args) {
    Object.keys(args).forEach(function (key) {
      if (attributes.indexOf(key) !== -1) {
        result.filter = result.filter || {};
        result.filter[key] = args[key];
      }

      // Limit arg
      if (key === 'offset' && args[key]) {
        result.offset = parseInt(args[key], 10);
      }

      if (key === 'skip' && args[key]) {
        result.index = parseInt(args[key], 10);
      }

      if (key === 'order' && args[key]) {
        if (args[key].indexOf('reverse:') === 0) {
          result.order = [args[key].substring(8), 'DESC'];
        } else {
          result.order = [args[key], 'ASC'];
        }
      }
    });

    var maxLimit = opts.maxLimit;

    if (maxLimit) {
      if (!result.offset) {
        result.offset = maxLimit;
      }

      if (result.offset > maxLimit) {
        result.offset = maxLimit;
      }
    }

    return result;
  }
}

/**
 * Build query
 *
 * @param seq
 * @param args
 * @param thinky
 * @param options
 * @returns {*}
 */
function buildQuery(seq, args, thinky) {
  var Query = seq;

  // Developer can overwrite query per node
  if (typeof args.query === 'function') {
    Query = args.query(seq, args, thinky);
  } else {
    if (args.filter && Object.keys(args.filter).length > 0) {
      Object.keys(args.filter).forEach(function (fieldName) {
        if ((0, _lodash.isFunction)(args.filter[fieldName])) {
          Query = Query.filter(args.filter[fieldName]);
          delete args.filter[fieldName];
        }
      });

      Query = Query.filter(args.filter);
    }

    if (args.order && args.order[1] === 'DESC') {
      Query = Query.orderBy(thinky.r.desc(args.order[0]));
    } else if (args.order && args.order[0]) {
      Query = Query.orderBy(args.order[0]);
    }

    if (args.offset) {
      Query = Query.slice(args.index, args.offset);
    }
  }

  return Query;
}

/**
 * Build Count
 * @param model
 * @param relatedIds
 * @param FK
 * @param opts
 * @returns {*|void|Promise}
 */
function buildCount(model, relatedIds, FK, opts) {
  var thinky = model._thinky;
  var r = thinky.r;
  opts.offset = false;

  if (relatedIds.length > 0 && FK) {
    var _seq = model.getAll(r.args((0, _lodash.uniq)(relatedIds)), { index: FK }).withFields(['id', FK].concat(Object.keys(opts.filter)));

    _seq = buildQuery(_seq, opts, thinky);

    return _seq.group(FK).ungroup().merge(function (selectionSet) {
      return { fullCount: selectionSet('reduction').count() };
    }).run();
  }

  var seq = buildQuery(model, opts, thinky);

  return seq.count().execute();
}

/**
 * Map count to result set
 * @param resultSet
 * @param countResults
 * @param FK
 * @returns {*}
 */
function mapCountToResultSet(resultSet, countResults, FK) {
  return resultSet.map(function (row) {
    if (Array.isArray(row)) {
      return row.map(function (innerRow) {
        var findCount = (0, _lodash.find)(countResults, { group: innerRow[FK] }) || { fullCount: 0 };
        innerRow.fullCount = findCount.fullCount;
        return innerRow;
      });
    } else {
      var findCount = (0, _lodash.find)(countResults, { group: row[FK] }) || { fullCount: 0 };
      row.fullCount = findCount.fullCount;
      return row;
    }
  });
}